# include <stdio.h>
# include <stdlib.h>
# include <string.h>
# include "mplib.h"
# include "mplibps.h"
# define EXECUTE mp_execute ( mp, command, strlen ( command ) )

// function declarations
double unscale ( int x ) ;
char * color_repr ( mp_color color, unsigned char color_model ) ;
char * ljoin_repr ( unsigned char ljoin ) ;
char * lcap_repr ( unsigned char lcap ) ;
void print_path_repr ( mp_knot * curnode ) ;

/* main function */

int main ( int argc, char * argv [] ) 
{
	// initalizing mplib
	MP mp ;
	MP_options * opt = mp_options () ;
	opt -> command_line = argv [ 1 ] ;
	opt -> noninteractive = 1 ;
	mp = mp_initialize ( opt ) ;
	if ( ! mp ) exit ( EXIT_FAILURE ) ;

	// variable declarations
	char * command ;
	mp_run_data * results ;
	mp_edge_object * curfig ;
	mp_graphic_object * curobj ;
	mp_fill_object * fillobj ;
	mp_stroked_object * strobj ;
	int fignum, objnum ;
	
	// commands to mplib
	command = "input plain ;" ; EXECUTE ;
	command = "beginfig ( 10 ) ; filldraw ( 0, 0 ) .. ( 2, 20 ) -- ( 10, 5 ) .. ( 3, 10 ) .. cycle ; endfig ; " ; EXECUTE ;
	command = "beginfig ( 20 ) ; draw ( 10, 0 ) .. ( 2, 2 ) -- ( 1, 5 ) .. ( 30, 0 ) ; " ; EXECUTE ;
	command = "draw ( 20, 0 ) -- ( 30, 10 ) ; endfig ; " ; EXECUTE ;
	command = 
		"beginfig ( 30 ) ; \
			path p ; p := dir ( -30 ) -- dir ( 90 ) -- dir ( 210 ) -- cycle ; \
			pen pentriangle ; pentriangle := makepen ( p ) ; \
			draw origin withpen pentriangle scaled 2 ; \
			draw ( p scaled 1cm ) withpen pentriangle scaled 4 ; \
			draw ( p scaled 2cm ) withpen pencircle scaled 8 ; \
		endfig ;" ; 
	EXECUTE ;
	command = "end ; " ; EXECUTE ;

	// getting and analysing the output
	printf ( "-------------------------------------------------------------------------------\n" ) ;
	printf ( "-------------------------------------------------------------------------------\n" ) ;
	results = mp_rundata ( mp ) ;
	curfig = results -> edges ;
	fignum = 1 ;
	while ( curfig != NULL )
	{
# define C(x) unscale ( curfig -> x )
		printf ( "Current figure #%d has internal figure number %d, \nand bounding box (%10.6f,%10.6f) to (%10.6f,%10.6f)\n\n", fignum, curfig -> charcode, C ( minx ) , C ( miny ), C ( maxx ), C ( maxy ) ) ;
# undef C
		
		curobj = curfig -> body ;
		objnum = 1 ;
		
# define OUTPUT_COMMON_PROPS(x) \
	printf ( "Color contents  : %s\n", color_repr ( x -> color, x -> color_model ) ) ; \
	printf ( "Line join style : %s\n", ljoin_repr ( x -> ljoin ) ) ; \
	printf ( "Miter limit     : %10.6f\n", unscale ( x -> miterlim ) ) ; \
	printf ( "Pen             : " ) ; print_path_repr ( x -> pen_p ) ; \
	printf ( "Path            :\n" ) ; print_path_repr ( x -> path_p ) ;
		
		while ( curobj != NULL )
		{
			switch ( curobj -> type )
			{
			case ( mp_fill_code ) :
				printf ( "Object #%d in figure #%d is a filled (possibly also stroked) object.\n", objnum, fignum ) ;
				fillobj = ( mp_fill_object * ) curobj ;
				OUTPUT_COMMON_PROPS ( fillobj ) // no semicolon after this macro
				printf ( "Reverse path    : " ) ; print_path_repr ( fillobj -> htap_p ) ;
				break ;
			case ( mp_stroked_code ) :
				printf ( "Object #%d in figure #%d is a purely stroked object.\n", objnum, fignum ) ;
				strobj = ( mp_stroked_object * ) curobj ;
				OUTPUT_COMMON_PROPS ( strobj ) // no semicolon after this macro
				printf ( "Line cap style  : %s\n", lcap_repr ( strobj -> lcap ) ) ;
				break ;
			default :
				printf ( "Object #%d in figure #%d is neither a filled nor stroked object.\n", objnum, fignum ) ;
				break ;
			}
			
			curobj = curobj -> next ; 
			++ objnum ;
			printf ( "-------------------------------------------------------------------------------\n" ) ;
		}
		curfig = curfig -> next ;
		++ fignum ;
		printf ( "-------------------------------------------------------------------------------\n" ) ;
	}
	
	mp_finish ( mp ) ;
}

/* other functions */

double unscale ( int x ) // returns the actual value stored as a scaled integer
{
	return ( x / 65536.0 ) ;
}

char * color_repr ( mp_color color, unsigned char color_model )
{
# define C(x) unscale ( color . x )
	static char output [ 100 ] ;
	switch ( color_model )
	{
	case ( mp_no_model ) :
		sprintf ( output, "No model" ) ;
		break ;
	case ( mp_grey_model ) :
		sprintf ( output, "Grey %.6f,%.6f", C ( a_val ) ) ;
		break ;
	case ( mp_rgb_model ) :
		sprintf ( output, "Red %.6f, Green %.6f, Blue %.6f", C ( a_val ), C ( b_val ), C ( c_val ) ) ;
		break ;
	case ( mp_cmyk_model ) :
		sprintf ( output, "Cyan %.6f, Magenta %.6f, Yellow %.6f, Black %.6f", C ( a_val ), C ( b_val ), C ( c_val ), C ( d_val ) ) ;
	}
	return output ;
# undef C
}

char * ljoin_repr ( unsigned char ljoin )
{
	switch ( ljoin )
	{
	case ( 0 ) : return "mitered" ;
	case ( 1 ) : return "round" ;
	case ( 2 ) : return "beveled" ;
	}
	return "unknown" ;
}

char * lcap_repr ( unsigned char lcap )
{
	switch ( lcap )
	{
	case ( 0 ) : return "butt" ;
	case ( 1 ) : return "round" ;
	case ( 2 ) : return "square" ;
	}
	return "unknown" ;
}

void print_path_repr ( mp_knot * curnode ) 
{
	if ( curnode == NULL ) 
	{
		printf ( "empty\n" ) ;
		return ;
	}
	
	int nodenum = 1 ;
	mp_knot * firstnode = curnode ;

# define C(x) unscale ( curnode -> x )
# define OUTPUT(a, b) printf ( "%10.6f,%10.6f", C ( a ), C ( b ) ) ;
	do
	{
		printf ( "Node #%02d: ", nodenum ) ;
		if ( curnode -> left_type != mp_endpoint )
		{
			printf ( "(" ) ;
			OUTPUT ( left_x, left_y ) // no semicolon after this macro
			printf ( ") " ) ; // note the space
		}
		OUTPUT ( x_coord, y_coord ) // no semicolon after this macro
		printf ( " " ) ;
		if ( curnode -> right_type != mp_endpoint )
		{
			printf ( "(" ) ;
			OUTPUT ( right_x, right_y ) // no semicolon after this macro
			printf ( ")" ) ; // note the space
		}
		printf ( "\n" ) ;
		curnode = curnode -> next ;
		++ nodenum ;
	} while ( curnode != firstnode ) ;
# undef OUTPUT
# undef C
}
